@@ -11,13 +11,15 @@
 Provides Counter and SubConter classes
 """
 
+import os
 import platform
 import re
 import sys
 import time
 
 from enlighten._basecounter import BaseCounter, PrintableCounter
-from enlighten._util import EnlightenWarning, format_time, warn_best_level
+from enlighten._util import (EnlightenWarning, FORMAT_MAP_SUPPORT, format_time,
+                             raise_from_none, warn_best_level)
 
 COUNTER_FMT = u'{desc}{desc_pad}{count:d} {unit}{unit_pad}' + \
               u'[{elapsed}, {rate:.2f}{unit_pad}{unit}/s]{fill}'
@@ -27,8 +29,8 @@ BAR_FMT = u'{desc}{desc_pad}{percentage:3.0f}%|{bar}| {count:{len_total}d}/{tota
 
 STATUS_FMT = u'{message}'
 
-# Even with cp65001, Windows doesn't seem to support all unicode characters
-if platform.system() == 'Windows':  # pragma: no cover(Windows)
+# Even with cp65001, Windows doesn't seem to support all unicode characters. Windows Terminal does
+if platform.system() == 'Windows' and not os.environ.get('WT_SESSION', None):  # pragma: no cover
     SERIES_STD = u' ▌█'
 else:
     SERIES_STD = u' ▏▎▍▌▋▊▉█'
@@ -42,9 +44,9 @@ except (AttributeError, TypeError):  # pragma: no cover(Non-standard Terminal)
     pass
 
 # Reserved fields
-COUNTER_FIELDS = {'count', 'desc', 'desc_pad', 'elapsed', 'rate', 'unit', 'unit_pad',
+COUNTER_FIELDS = {'count', 'desc', 'desc_pad', 'elapsed', 'interval', 'rate', 'unit', 'unit_pad',
                   'bar', 'eta', 'len_total', 'percentage', 'total', 'fill'}
-RE_SUBCOUNTER_FIELDS = re.compile(r'(?:count|percentage|eta|rate)_\d+')
+RE_SUBCOUNTER_FIELDS = re.compile(r'(?:count|percentage|eta|interval|rate)_\d+')
 
 
 class SubCounter(BaseCounter):
@@ -74,7 +76,7 @@ class SubCounter(BaseCounter):
         Args:
             color(str): Series color as a string or RGB tuple see :ref:`Series Color <series_color>`
             count(int): Initial count (Default: 0)
-            all_fields(bool): Populate ``rate`` and ``eta`` fields (Default: False)
+            all_fields(bool): Populate ``rate``, ``interval``, and ``eta`` fields (Default: False)
         """
 
         if parent.count - parent.subcount - count < 0:
@@ -292,7 +294,8 @@ class Counter(PrintableCounter):
         - desc(:py:class:`str`) - Value of ``desc``
         - desc_pad(:py:class:`str`) - A single space if ``desc`` is set, otherwise empty
         - elapsed(:py:class:`str`) - Time elapsed since instance was created
-        - rate(:py:class:`float`) - Average increments per second since instance was created
+        - interval(:py:class:`float`) - Average seconds per iteration (inverse of rate)
+        - rate(:py:class:`float`) - Average iterations per second since instance was created
         - unit(:py:class:`str`) - Value of ``unit``
         - unit_pad(:py:class:`str`) - A single space if ``unit`` is set, otherwise empty
 
@@ -327,7 +330,8 @@ class Counter(PrintableCounter):
         ``all_fields`` set to :py:data:`True`:
 
         - eta_n (:py:class:`str`) - Estimated time to completion (``bar_format`` only)
-        - rate_n (:py:class:`float`) - Average increments per second since parent was created
+        - interval_n(:py:class:`float`) - Average seconds per iteration (inverse of rate)
+        - rate_n (:py:class:`float`) - Average iterations per second since parent was created
 
         User-defined fields:
 
@@ -447,7 +451,7 @@ class Counter(PrintableCounter):
         Each subcounter in the list will be in a tuple of (subcounter, percentage)
         Fields in the dictionary are addressed in the Format documentation of this class
 
-        When `bar_fields` is False, only subcounter count and rate fields are set.
+        When `bar_fields` is False, only subcounter count, interval, and rate fields are set.
         percentage will be set to 0.0
         """
 
@@ -456,7 +460,7 @@ class Counter(PrintableCounter):
 
         for num, subcounter in enumerate(self._subcounters, 1):
 
-            fields['count_{0}'.format(num)] = subcounter.count
+            fields['count_%d' % num] = subcounter.count
 
             if self.total and bar_fields:
                 subPercentage = subcounter.count / float(self.total)
@@ -464,30 +468,32 @@ class Counter(PrintableCounter):
                 subPercentage = 0.0
 
             if bar_fields:
-                fields['percentage_{0}'.format(num)] = subPercentage * 100
+                fields['percentage_%d' % num] = subPercentage * 100
 
             # Save in tuple: count, percentage
             subcounters.append((subcounter, subPercentage))
 
             if subcounter.all_fields:
 
-                interations = abs(subcounter.count - subcounter.start_count)
+                interations = float(abs(subcounter.count - subcounter.start_count))
 
                 if elapsed:
                     # Use float to force to float in Python 2
-                    rate = fields['rate_{0}'.format(num)] = interations / float(elapsed)
+                    rate = fields['rate_%d' % num] = interations / elapsed
                 else:
-                    rate = fields['rate_{0}'.format(num)] = 0.0
+                    rate = fields['rate_%d' % num] = 0.0
+
+                fields['interval_%d' % num] = rate ** -1 if rate else 0.0
 
                 if not bar_fields:
                     continue
 
                 if self.total == 0:
-                    fields['eta_{0}'.format(num)] = u'00:00'
+                    fields['eta_%d' % num] = u'00:00'
                 elif rate:
-                    fields['eta_{0}'.format(num)] = format_time((self.total - interations) / rate)
+                    fields['eta_%d' % num] = format_time((self.total - interations) / rate)
                 else:
-                    fields['eta_{0}'.format(num)] = u'?'
+                    fields['eta_%d' % num] = u'?'
 
         return subcounters, fields
 
@@ -506,7 +512,7 @@ class Counter(PrintableCounter):
 
         width = width or self.manager.width
 
-        iterations = abs(self.count - self.start_count)
+        iterations = float(abs(self.count - self.start_count))
 
         fields = self.fields.copy()
         fields.update(self._fields)
@@ -537,10 +543,11 @@ class Counter(PrintableCounter):
         # Get rate. Elapsed could be 0 if counter was not updated and has a zero total.
         if elapsed:
             # Use iterations so a counter running backwards is accurate
-            fields['rate'] = iterations / elapsed
+            rate = fields['rate'] = iterations / elapsed
         else:
-            fields['rate'] = 0.0
+            rate = fields['rate'] = 0.0
 
+        fields['interval'] = rate ** -1 if rate else 0.0
         # Only process bar if total was given and n doesn't exceed total
         if self.total is not None and self.count <= self.total:
 
@@ -556,9 +563,9 @@ class Counter(PrintableCounter):
                 percentage = self.count / float(self.total)
 
                 # Get eta
-                if fields['rate']:
+                if rate:
                     # Use iterations so a counter running backwards is accurate
-                    fields['eta'] = format_time((self.total - iterations) / fields['rate'])
+                    fields['eta'] = format_time((self.total - iterations) / rate)
                 else:
                     fields['eta'] = u'?'
 
@@ -575,9 +582,12 @@ class Counter(PrintableCounter):
 
             # Partially format
             try:
-                rtn = self.bar_format.format(**fields)
+                if FORMAT_MAP_SUPPORT:
+                    rtn = self.bar_format.format_map(fields)
+                else:  # pragma: no cover
+                    rtn = self.bar_format.format(**fields)
             except KeyError as e:
-                raise ValueError('%r specified in format, but not provided' % e.args[0])
+                raise_from_none(ValueError('%r specified in format, but not provided' % e.args[0]))
 
             # Format the bar
             if self.offset is None:
@@ -615,9 +625,12 @@ class Counter(PrintableCounter):
             fields['count_0'] = self.count - sum(sub[0].count for sub in subcounters)
 
         try:
-            rtn = self.counter_format.format(**fields)
+            if FORMAT_MAP_SUPPORT:
+                rtn = self.counter_format.format_map(fields)
+            else:  # pragma: no cover
+                rtn = self.counter_format.format(**fields)
         except KeyError as e:
-            raise ValueError('%r specified in format, but not provided' % e.args[0])
+            raise_from_none(ValueError('%r specified in format, but not provided' % e.args[0]))
 
         return self._fill_text(rtn, width, offset=self.offset)
 
@@ -647,7 +660,8 @@ class Counter(PrintableCounter):
     Args:
         color(str): Series color as a string or RGB tuple see :ref:`Series Color <series_color>`
         count(int): Initial count (Default: 0)
-        all_fields(bool): Populate ``rate`` and ``eta`` formatting fields (Default: False)
+        all_fields(bool): Populate ``rate``, ``interval``, and ``eta``
+            formatting fields (Default: False)
 
     Returns:
         :py:class:`SubCounter`: Subcounter instance
