@@ -13,14 +13,16 @@ Provides Manager class
 
 import atexit
 from collections import OrderedDict
+import multiprocessing
 import signal
 import sys
 import threading
 import time
 
+from blessed import Terminal
+
 from enlighten._counter import Counter
 from enlighten._statusbar import StatusBar
-from enlighten._terminal import Terminal
 
 RESIZE_SUPPORTED = hasattr(signal, 'SIGWINCH')
 
@@ -38,7 +40,7 @@ class Manager(object):
         enabled(bool): Status (Default: True)
         no_resize(bool): Disable resizing support
         threaded(bool): When True resize handling is deferred until next write (Default: False
-            unless multiple threads are detected)
+            unless multiple threads or multiple processes are detected)
         kwargs(Dict[str, Any]): Any additional :py:term:`keyword arguments<keyword argument>`
             will be used as default values when :py:meth:`counter` is called.
 
@@ -73,7 +75,7 @@ class Manager(object):
         self.enabled = kwargs.get('enabled', True)  # Double duty for counters
         self.no_resize = kwargs.pop('no_resize', False)
         self.set_scroll = kwargs.pop('set_scroll', True)
-        self.threaded = kwargs.pop('threaded', threading.active_count() > 1)
+        self.threaded = kwargs.pop('threaded', None)  # Defer evaluation
         self.term = Terminal(stream=self.stream)
 
         # Set up companion stream
@@ -99,6 +101,8 @@ class Manager(object):
             self.companion_term = None
 
         self.autorefresh = []
+        self._buffer = []
+        self._companion_buffer = []
         self.height = self.term.height
         self.process_exit = False
         self.refresh_lock = False
@@ -285,35 +289,35 @@ class Manager(object):
         """
 
         # Make sure only one resize handler is running
-        try:
-            assert self.resize_lock
-        except AssertionError:
+        if self.resize_lock:
+            return
 
-            self.resize_lock = True
-            term = self.term
+        self.resize_lock = True
+        buffer = self._buffer
+        term = self.term
 
-            term.clear_cache()
-            newHeight = term.height
-            newWidth = term.width
+        oldHeight = self.height
+        newHeight = self.height = term.height
+        newWidth = term.width
 
-            if newHeight < self.height:
-                term.move_to(0, max(0, newHeight - self.scroll_offset))
-                self.stream.write(u'\n' * (2 * max(self.counters.values())))
-            elif newHeight > self.height and self.threaded:
-                term.move_to(0, newHeight)
-                self.stream.write(u'\n' * (self.scroll_offset - 1))
+        if newHeight < oldHeight:
+            buffer.append(term.move(max(0, newHeight - self.scroll_offset), 0))
+            buffer.append(u'\n' * (2 * max(self.counters.values())))
+        elif newHeight > oldHeight and self.threaded:
+            buffer.append(term.move(newHeight, 0))
+            buffer.append(u'\n' * (self.scroll_offset - 1))
 
-            term.move_to(0, max(0, newHeight - self.scroll_offset))
-            self.stream.write(term.clear_eos)
+        buffer.append(term.move(max(0, newHeight - self.scroll_offset), 0))
+        buffer.append(term.clear_eos)
 
-            self.width = newWidth
-            self._set_scroll_area(force=True)
+        self.width = newWidth
+        self._set_scroll_area(force=True)
 
-            for counter in self.counters:
-                counter.refresh(flush=False)
-            self._flush_streams()
+        for counter in self.counters:
+            counter.refresh(flush=False)
+        self._flush_streams()
 
-            self.resize_lock = False
+        self.resize_lock = False
 
     def _set_scroll_area(self, force=False):
         """
@@ -339,40 +343,58 @@ class Manager(object):
         if not self.process_exit:
             atexit.register(self._at_exit)
             if not self.no_resize and RESIZE_SUPPORTED:
+                if self.threaded is None:
+                    self.threaded = (
+                        threading.active_count() > 1  # Multiple threads
+                        or multiprocessing.active_children()  # Main process with children
+                        or multiprocessing.current_process().name != 'MainProcess'  # Child process
+                    )
                 signal.signal(signal.SIGWINCH, self._stage_resize)
             self.process_exit = True
 
         if self.set_scroll:
 
+            buffer = self._buffer
             term = self.term
-            newHeight = term.height
-            scrollPosition = max(0, newHeight - self.scroll_offset)
+            scrollPosition = max(0, self.height - self.scroll_offset)
 
-            if force or use_new or newHeight != self.height:
-                self.height = newHeight
+            if force or use_new:
 
                 # Add line feeds so we don't overwrite existing output
                 if use_new:
-                    term.move_to(0, max(0, newHeight - oldOffset))
-                    self.stream.write(u'\n' * (newOffset - oldOffset))
+                    buffer.append(term.move(max(0, self.height - oldOffset), 0))
+                    buffer.append(u'\n' * (newOffset - oldOffset))
 
                 # Reset scroll area
-                self.term.change_scroll(scrollPosition)
+                buffer.append(term.hide_cursor)
+                buffer.append(term.csr(0, scrollPosition))
 
             # Always reset position
-            term.move_to(0, scrollPosition)
+            buffer.append(term.move(scrollPosition, 0))
             if self.companion_term is not None:
-                self.companion_term.move_to(0, scrollPosition)
+                self._companion_buffer.append(term.move(scrollPosition, 0))
 
     def _flush_streams(self):
         """
         Convenience method for flushing streams
         """
 
+        buffer = self._buffer
+        companion_buffer = self._companion_buffer
+
+        if buffer:
+            self.stream.write(u''.join(buffer))
+
         self.stream.flush()
+
         if self.companion_stream is not None:
+            if companion_buffer:
+                self.companion_stream.write(u''.join(companion_buffer))
             self.companion_stream.flush()
 
+        del buffer[:]  # Python 2.7 does not support list.clear()
+        del companion_buffer[:]
+
     def _at_exit(self):
         """
         Resets terminal to normal configuration
@@ -383,13 +405,14 @@ class Manager(object):
 
         try:
             term = self.term
+            buffer = self._buffer
 
             if self.set_scroll:
-                term.reset()
-            else:
-                term.move_to(0, term.height)
+                buffer.append(self.term.normal_cursor)
+                buffer.append(self.term.csr(0, self.height - 1))
 
-            self.term.feed()
+            buffer.append(term.move(term.height, 0))
+            buffer.append(term.cud1 or u'\n')
 
             self._flush_streams()
 
@@ -432,8 +455,9 @@ class Manager(object):
         if not self.enabled:
             return
 
+        buffer = self._buffer
         term = self.term
-        stream = self.stream
+        height = term.height
         positions = self.counters.values()
 
         if not self.no_resize and RESIZE_SUPPORTED:
@@ -442,20 +466,22 @@ class Manager(object):
         try:
             for num in range(self.scroll_offset - 1, 0, -1):
                 if num not in positions:
-                    term.move_to(0, term.height - num)
-                    stream.write(term.clear_eol)
+                    buffer.append(term.move(height - num, 0))
+                    buffer.append(term.clear_eol)
 
         finally:
 
+            # Reset terminal
             if self.set_scroll:
-
-                self.term.reset()
-
+                buffer.append(term.normal_cursor)
+                buffer.append(term.csr(0, self.height - 1))
                 if self.companion_term:
-                    self.companion_term.reset()
+                    self._companion_buffer.extend((term.normal_cursor,
+                                                   term.csr(0, self.height - 1),
+                                                   term.move(height, 0)))
 
-            else:
-                term.move_to(0, term.height)
+            # Re-home cursor
+            buffer.append(term.move(height, 0))
 
             self.process_exit = False
             self.enabled = False
@@ -464,7 +490,7 @@ class Manager(object):
 
         # Feed terminal if lowest position isn't cleared
         if 1 in positions:
-            term.feed()
+            buffer.append(term.cud1 or '\n')
 
         self._flush_streams()
 
@@ -504,9 +530,10 @@ class Manager(object):
             output = output(**kwargs)
 
         try:
-            term.move_to(0, term.height - position)
-            # Include \r and term call to cover most conditions
-            self.stream.write(u'\r' + term.clear_eol + output)
+            self._buffer.extend((term.move(self.height - position, 0),
+                                 u'\r',
+                                 term.clear_eol,
+                                 output))
 
         finally:
             # Reset position and scrolling
@@ -525,22 +552,17 @@ class Manager(object):
         Refresh any bars specified for auto-refresh
         """
 
-        # Make sure this is only running once
-        try:
-            assert self.refresh_lock
-        except AssertionError:
+        self.refresh_lock = True
+        current_time = time.time()
 
-            self.refresh_lock = True
-            current_time = time.time()
+        for counter in self.autorefresh:
 
-            for counter in self.autorefresh:
+            if counter in exclude or counter.min_delta > current_time - counter.last_update:
+                continue
 
-                if counter in exclude or counter.min_delta > current_time - counter.last_update:
-                    continue
+            counter.refresh()
 
-                counter.refresh()
-
-            self.refresh_lock = False
+        self.refresh_lock = False
 
 
 def get_manager(stream=None, counterclass=Counter, **kwargs):
